'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _objectEntries(obj) {
  var entries = [];
  var keys = Object.keys(obj);

  for (var k = 0; k < keys.length; k++) entries.push([keys[k], obj[keys[k]]]);

  return entries;
}

var Model = /*#__PURE__*/function () {
  /**
   * @param {string} type
   * @param {string} id
   */
  function Model(type, id) {
    this.id = id;
    this._type = type;
    this._attributes = [];
    this._dependents = [];
    this._relationships = [];
    this._links = this.links = {};
    this._relationshipLinks = {};
    this._meta = this.meta = {};
    this._relationshipMeta = {};
    this._destroyed = false;
  }
  /**
   * @param {string} type
   * @param {string} id
   * @param {string} key
   */


  var _proto = Model.prototype;

  _proto._addDependence = function _addDependence(type, id, key) {
    var found = this._dependents.find(function (dependent) {
      return dependent.id === id && dependent.type === type && dependent.relation === key;
    });

    if (typeof found === 'undefined') {
      this._dependents.push({
        id: id,
        type: type,
        relation: key
      });
    }
  }
  /**
   * @param {string} type
   * @param {string} id
   */
  ;

  _proto._removeDependence = function _removeDependence(type, id) {
    var _this = this;

    this._dependents.forEach(function (value, index) {
      if (value.id === id && value.type === type) {
        _this._dependents.splice(index, 1);
      }
    });
  }
  /**
   * @param {string} type
   * @param {string} id
   * @param {string} relationshipName
   */
  ;

  _proto.removeRelationship = function removeRelationship(type, id, relationshipName) {
    var _this2 = this;

    this._removeDependence(type, id);

    if (Array.isArray(this[relationshipName])) {
      this[relationshipName].forEach(function (value, index) {
        if (value.id === id && value._type === type) {
          _this2[relationshipName].splice(index, 1);
        }
      });
    } else if (this[relationshipName].id === id) {
      this[relationshipName] = null;
    }
  }
  /**
   * @param {object} options
   * @param {Array} options.attributes
   * @param {Array} options.relationships
   * @param {Array} options.links
   * @param {Array} options.meta
   *
   * @returns {object}
   */
  ;

  _proto.serialize = function serialize(options) {
    var _this3 = this;

    if (options === void 0) {
      options = {};
    }

    var relationshipIdentifier = function relationshipIdentifier(model) {
      return {
        type: model._type,
        id: model.id
      };
    };

    var response = {
      data: {
        type: this._type
      }
    };
    var _options = options,
        _options$attributes = _options.attributes,
        attributes = _options$attributes === void 0 ? this._attributes : _options$attributes,
        _options$relationship = _options.relationships,
        relationships = _options$relationship === void 0 ? this._relationships : _options$relationship,
        links = _options.links,
        meta = _options.meta;

    if (typeof this.id !== 'undefined') {
      response.data.id = this.id;
    }

    if (attributes.length !== 0) {
      response.data.attributes = {};
    }

    if (relationships.length !== 0) {
      response.data.relationships = {};
    }

    attributes.forEach(function (key) {
      response.data.attributes[key] = _this3[key];
    });
    relationships.forEach(function (key) {
      if (!_this3[key]) {
        response.data.relationships[key] = {
          data: null
        };
      } else if (Array.isArray(_this3[key])) {
        response.data.relationships[key] = {
          data: _this3[key].map(function (relationship) {
            return relationshipIdentifier(relationship);
          })
        };
      } else {
        response.data.relationships[key] = {
          data: relationshipIdentifier(_this3[key])
        };
      }

      if (_this3._relationshipLinks[key]) {
        response.data.relationships[key].links = _this3._relationshipLinks[key];
      }

      if (_this3._relationshipMeta[key]) {
        response.data.relationships[key].meta = _this3._relationshipMeta[key];
      }
    });

    if (Object.keys(this._links).length !== 0) {
      if (typeof links === 'undefined') {
        response.data.links = this._links;
      } else if (Array.isArray(links) && links.length !== 0) {
        response.data.links = {};
        links.forEach(function (key) {
          response.data.links[key] = _this3._links[key];
        });
      }
    }

    if (Object.keys(this._meta).length !== 0) {
      if (typeof meta === 'undefined') {
        response.data.meta = this._meta;
      } else if (Array.isArray(meta) && meta.length !== 0) {
        response.data.meta = {};
        meta.forEach(function (key) {
          response.data.meta[key] = _this3._meta[key];
        });
      }
    }

    return response;
  }
  /**
   * @param {string} attributeName
   * @param {*} value
   */
  ;

  _proto.setAttribute = function setAttribute(attributeName, value) {
    if (typeof this[attributeName] === 'undefined') {
      this._attributes.push(attributeName);
    }

    this[attributeName] = value;
  }
  /**
   * @param {string} relationshipName
   * @param {Model[]} models
   */
  ;

  _proto.setRelationship = function setRelationship(relationshipName, models) {
    if (typeof this[relationshipName] === 'undefined') {
      this._relationships.push(relationshipName);

      this[relationshipName] = models;
    } else if (Array.isArray(this[relationshipName])) {
      this[relationshipName].push(models);
    } else {
      this[relationshipName] = models;
    }
  };

  return Model;
}();

var Store = /*#__PURE__*/function () {
  function Store() {
    this.graph = {};
    this.order = {};
  }
  /**
   * @param {Model} model
   */


  var _proto2 = Store.prototype;

  _proto2.destroy = function destroy(model) {
    var _this4 = this;

    model._destroyed = true;

    model._dependents.forEach(function (dependent) {
      _this4.graph[dependent.type][dependent.id].removeRelationship(model._type, model.id, dependent.relation);
    });

    delete this.graph[model._type][model.id];

    this.order[model._type].splice(this.order[model._type].indexOf(model.id), 1);
  }
  /**
   * @param {string} type
   * @param {string} id
   *
   * @returns {?Model}
   */
  ;

  _proto2.find = function find(type, id) {
    if (!this.graph[type] || !this.graph[type][id]) {
      return null;
    }

    return this.graph[type][id];
  }
  /**
   * @param {string} type
   *
   * @returns {Model[]}
   */
  ;

  _proto2.findAll = function findAll(type) {
    var _this5 = this;

    if (!this.graph[type]) {
      return [];
    }

    return this.order[type].map(function (modelId) {
      return _this5.graph[type][modelId];
    });
  };

  _proto2.reset = function reset() {
    this.graph = {};
    this.order = {};
  }
  /**
   * @param  {string} type
   * @param  {string} id
   *
   * @returns {Model}
   */
  ;

  _proto2.initModel = function initModel(type, id) {
    this.graph[type] = this.graph[type] || {};
    this.order[type] = this.order[type] || [];
    this.graph[type][id] = this.graph[type][id] || new Model(type, id);
    var currentOrderIndex = this.order[type].indexOf(id);

    if (currentOrderIndex === -1) {
      this.order[type].push(id);
    } else {
      // Remove the id from the current order and add it to the bottom
      this.order[type].splice(currentOrderIndex, 1);
      this.order[type].push(id);
    }

    return this.graph[type][id];
  }
  /**
   * @param  {object} record
   *
   * @returns {Model}
   */
  ;

  _proto2.syncRecord = function syncRecord(record) {
    var _this6 = this;

    var model = this.initModel(record.type, record.id);

    var findOrInit = function findOrInit(resource) {
      if (!_this6.find(resource.type, resource.id)) {
        var placeHolderModel = _this6.initModel(resource.type, resource.id);

        placeHolderModel._placeHolder = true;
      }

      return _this6.graph[resource.type][resource.id];
    };

    delete model._placeHolder;

    if (record.attributes) {
      _objectEntries(record.attributes).forEach(function (_ref) {
        var key = _ref[0],
            attribute = _ref[1];

        if (model._attributes.indexOf(key) === -1) {
          model._attributes.push(key);
        }

        model[key] = attribute;
      });
    }

    if (record.links) {
      model._links = model.links = record.links;
    }

    if (record.meta) {
      model._meta = model.meta = record.meta;
    }

    if (record.relationships) {
      _objectEntries(record.relationships).forEach(function (_ref2) {
        var key = _ref2[0],
            relationship = _ref2[1];

        if (typeof relationship.data !== 'undefined') {
          if (model._relationships.indexOf(key) === -1) {
            model._relationships.push(key);
          }

          if (relationship.data === null) {
            model[key] = null;
          } else if (Array.isArray(relationship.data)) {
            model[key] = relationship.data.map(function (relationship) {
              return findOrInit(relationship);
            });
            model[key].forEach(function (record) {
              record._addDependence(model._type, model.id, key);
            });
          } else {
            model[key] = findOrInit(relationship.data);

            model[key]._addDependence(model._type, model.id, key);
          }
        }

        if (relationship.links) {
          model._relationshipLinks[key] = relationship.links;
        }

        if (relationship.meta) {
          model._relationshipMeta[key] = relationship.meta;
        }
      });
    }

    return model;
  }
  /**
   * @param {object} payload
   * @param {object} options
   * @param {boolean} options.topLevel
   *
   * @returns {Model|Model[]}
   */
  ;

  _proto2.sync = function sync(payload, options) {
    var _this7 = this;

    if (payload === void 0) {
      payload = {};
    }

    if (options === void 0) {
      options = {};
    }

    var _payload = payload,
        data = _payload.data,
        meta = _payload.meta,
        links = _payload.links,
        jsonapi = _payload.jsonapi,
        errors = _payload.errors,
        included = _payload.included;
    var _options2 = options,
        _options2$topLevel = _options2.topLevel,
        topLevel = _options2$topLevel === void 0 ? false : _options2$topLevel;
    var response = {};

    if (typeof meta !== 'undefined') {
      response.meta = meta;
    }

    if (typeof links !== 'undefined') {
      response.links = links;
    }

    if (typeof jsonapi !== 'undefined') {
      response.jsonapi = jsonapi;
    }

    if (typeof errors !== 'undefined') {
      response.errors = errors;
    }

    if (typeof included !== 'undefined') {
      if (Array.isArray(included)) {
        included.forEach(function (record) {
          _this7.syncRecord(record);
        });
      }
    }

    if (typeof data !== 'undefined' && data !== null) {
      if (Array.isArray(data)) {
        response.data = data.map(function (record) {
          return _this7.syncRecord(record);
        });
      } else {
        response.data = this.syncRecord(data);
      }
    } else {
      response.data = [];
    }

    if (topLevel || typeof errors !== 'undefined') {
      return response;
    }

    return response.data;
  };

  return Store;
}();

exports.Model = Model;
exports.Store = Store;
//# sourceMappingURL=index.js.map
